@@ -1,9 +1,11 @@
 """
 Classes for generating lists of photons
 """
+
 import h5py
 import numpy as np
 from soxs import __version__ as soxs_version
+from soxs.constants import erg_per_keV
 from soxs.utils import parse_prng
 from tqdm.auto import tqdm
 from unyt.array import unyt_array
@@ -116,6 +118,7 @@ def make_photons(
     velocity_fields=None,
     bulk_velocity=None,
     observer="external",
+    fields_to_keep=None,
 ):
     r"""
     Write a photon list dataset to disk from a yt data source and assuming a
@@ -171,6 +174,14 @@ def make_photons(
         A 3-element array or list specifying the local velocity frame of
         reference. If not a :class:`~yt.units.yt_array.YTArray`, it is assumed
         to have units of km/s. Default: [0.0, 0.0, 0.0] km/s.
+    observer : string, optional
+        Controls whether the source is at a large distance, in which case
+        the observer sits far outside the source ("external"), or if the
+        observer is inside the X-ray-emitting source ("internal"), such as a
+        galaxy. Default: "external"
+    fields_to_keep : list of tuples
+        A list of fields to add to the HDF5 photon list in addition to the cell
+        or particle positions, velocities, and sizes. Default: None
 
     Returns
     -------
@@ -179,13 +190,16 @@ def make_photons(
 
     Examples
     --------
-    >>> thermal_model = pyxsim.CIESourceModel(0.1, 10.0, 1000, 0.3)
+    >>> thermal_model = pyxsim.CIESourceModel("apec", 0.1, 10.0, 1000, 0.3)
     >>> redshift = 0.05
     >>> area = 6000.0 # assumed here in cm**2
-    >>> time = 2.0e5 # assumed here in seconds
+    >>> exp_time = 2.0e5 # assumed here in seconds
     >>> sp = ds.sphere("c", (500., "kpc"))
-    >>> n_photons, n_cells = pyxsim.make_photons(sp, redshift, area,
-    ...                                          time, thermal_model)
+    >>> fields_to_keep = [("gas", "density"), ("gas", "temperature")]
+    >>> n_photons, n_cells = pyxsim.make_photons(
+    ...     sp, redshift, area, exp_time, thermal_model,
+    ...     fields_to_keep=fields_to_keep
+    ... )
     """
     ds = data_source.ds
 
@@ -194,8 +208,10 @@ def make_photons(
 
     if comm.size > 1:
         photon_file = f"{photon_prefix}.{comm.rank:04d}.h5"
+        photon_files = [f"{photon_prefix}.{i:04d}.h5" for i in range(comm.size)]
     else:
         photon_file = f"{photon_prefix}.h5"
+        photon_files = [photon_file]
 
     if parameters is None:
         parameters = {}
@@ -289,7 +305,7 @@ def make_photons(
     else:
         mylog.info("The observer is internal to the source.")
 
-    local_exp_time = parameters["fid_exp_time"].v
+    local_exp_time = parameters["fid_exp_time"].to_value("s")
     D_A = parameters["fid_d_a"].to_value("cm")
     dist_fac = 1.0 / (4.0 * np.pi)
     if observer == "external":
@@ -310,6 +326,12 @@ def make_photons(
     if velocity_fields is not None:
         v_fields = velocity_fields
 
+    fields_store = []
+    if fields_to_keep is not None:
+        for field in fields_to_keep:
+            fd = ds._get_field_info(field)
+            fields_store.append(fd.name)
+
     if p_fields[0] == ("index", "x"):
         parameters["data_type"] = "cells"
     else:
@@ -327,6 +349,7 @@ def make_photons(
     info.attrs["dataset"] = str(data_source.ds)
     info.attrs["data_source"] = str(data_source)
     info.attrs["source_model"] = repr(source_model)
+    info.attrs["filenames"] = photon_files
 
     # Parameters
 
@@ -337,7 +360,7 @@ def make_photons(
     p.create_dataset("hubble", data=parameters["hubble"])
     p.create_dataset("omega_matter", data=parameters["omega_matter"])
     p.create_dataset("omega_lambda", data=parameters["omega_lambda"])
-    p.create_dataset("fid_d_a", data=float(parameters["fid_d_a"]))
+    p.create_dataset("fid_d_a", data=parameters["fid_d_a"].to_value("Mpc"))
     p.create_dataset("data_type", data=parameters["data_type"])
     p.create_dataset("observer", data=parameters["observer"])
     p.create_dataset("center", data=parameters["center"].d)
@@ -352,6 +375,9 @@ def make_photons(
     p_size = init_chunk
 
     cell_fields = ["x", "y", "z", "vx", "vy", "vz", "num_photons", "dx"]
+    if len(fields_store) > 0:
+        for field in fields_store:
+            cell_fields.append(field[1])
 
     d = f.create_group("data")
     for field in cell_fields + ["energy"]:
@@ -370,11 +396,9 @@ def make_photons(
     f.flush()
 
     for chunk in parallel_objects(data_source.chunks([], "io")):
-
         chunk_data = source_model.process_data("photons", chunk, spectral_norm)
 
         if chunk_data is not None:
-
             chunk_nc, number_of_photons, idxs, energies = chunk_data
 
             chunk_nph = np.sum(number_of_photons)
@@ -393,12 +417,16 @@ def make_photons(
                     p_size *= 2
                 d["energy"].resize((p_size,))
 
+            if w_field is None:
+                dx = 0.0
+            else:
+                dx = chunk[w_field][idxs].to_value("kpc")
             for i, ax in enumerate("xyz"):
                 pos = chunk[p_fields[i]][idxs].to_value("kpc")
                 # Fix photon coordinates for regions crossing a periodic boundary
                 if ds.periodicity[i]:
-                    tfl = pos < le[i]
-                    tfr = pos > re[i]
+                    tfl = pos + dx < le[i]
+                    tfr = pos - dx > re[i]
                     pos[tfl] += dw[i]
                     pos[tfr] -= dw[i]
 
@@ -414,9 +442,10 @@ def make_photons(
             if w_field is None:
                 d["dx"][c_offset : c_offset + chunk_nc] = 0.0
             else:
-                d["dx"][c_offset : c_offset + chunk_nc] = chunk[w_field][idxs].to_value(
-                    "kpc"
-                )
+                d["dx"][c_offset : c_offset + chunk_nc] = dx
+
+            for field in fields_store:
+                d[field[1]][c_offset : c_offset + chunk_nc] = chunk[field][idxs]
 
             n_cells += chunk_nc
             n_photons += chunk_nph
@@ -463,7 +492,6 @@ def _project_photons(
     save_los=False,
     prng=None,
 ):
-
     from yt.funcs import ensure_numpy_array
 
     prng = parse_prng(prng)
@@ -490,9 +518,11 @@ def _project_photons(
     if comm.size > 1:
         photon_file = f"{photon_prefix}.{comm.rank:04d}.h5"
         event_file = f"{event_prefix}.{comm.rank:04d}.h5"
+        event_files = [f"{event_prefix}.{i:04d}.h5" for i in range(comm.size)]
     else:
         photon_file = f"{photon_prefix}.h5"
         event_file = f"{event_prefix}.h5"
+        event_files = [event_file]
 
     sky_center = ensure_numpy_array(sky_center)
 
@@ -550,15 +580,13 @@ def _project_photons(
 
     d = f["data"]
 
-    D_A = p["fid_d_a"][()] * 1.0e3
+    D_A = p["fid_d_a"][()] * 1.0e3  # assumes that the value from the file is in Mpc
 
     if d["energy"].size == 0:
-
         mylog.warning("No photons are in file %s, so I am done.", photon_file)
         n_events = 0
 
     else:
-
         fe = h5py.File(event_file, "w")
 
         ie = fe.create_group("info")
@@ -566,10 +594,14 @@ def _project_photons(
         ie.attrs["yt_version"] = yt_version
         ie.attrs["soxs_version"] = soxs_version
         ie.attrs["photon_file"] = photon_file
+        ie.attrs["filenames"] = event_files
+
+        exp_time = float(p["fid_exp_time"][()])
+        area = float(p["fid_area"][()])
 
         pe = fe.create_group("parameters")
-        pe.create_dataset("exp_time", data=float(p["fid_exp_time"][()]))
-        pe.create_dataset("area", data=float(p["fid_area"][()]))
+        pe.create_dataset("exp_time", data=exp_time)
+        pe.create_dataset("area", data=area)
         pe.create_dataset("sky_center", data=sky_center)
         pe.create_dataset("observer", data=observer)
         pe.create_dataset("no_shifting", data=int(no_shifting))
@@ -611,8 +643,11 @@ def _project_photons(
             leave=True, total=n_cells, desc="Projecting photons from cells/particles "
         )
 
-        for start_c in range(0, n_cells, cell_chunk):
+        flux = 0.0
+        emin = 1.0e99
+        emax = -1.0e99
 
+        for start_c in range(0, n_cells, cell_chunk):
             end_c = min(start_c + cell_chunk, n_cells)
 
             n_ph = d["num_photons"][start_c:end_c]
@@ -656,15 +691,13 @@ def _project_photons(
 
             if absorb_model is None:
                 det = np.ones(eobs.size, dtype="bool")
-                num_det = eobs.size
+                num_det = np.int64(eobs.size)
             else:
                 det = absorb_model.absorb_photons(eobs, prng=prng)
-                num_det = det.sum()
+                num_det = np.int64(det.sum())
 
             if num_det > 0:
-
                 if observer == "external":
-
                     if data_type == "particles":
                         dx *= 0.5
 
@@ -700,7 +733,6 @@ def _project_photons(
                         pixel_to_cel(xsky, ysky, sky_center)
 
                 elif observer == "internal":
-
                     xsky, ysky, los = scatter_events_allsky(
                         data_type,
                         kernel,
@@ -726,6 +758,10 @@ def _project_photons(
                 de["xsky"][e_offset : e_offset + num_det] = xsky
                 de["ysky"][e_offset : e_offset + num_det] = ysky
                 de["eobs"][e_offset : e_offset + num_det] = eobs[det]
+                flux += eobs[det].sum()
+                emin = min(eobs[det].min(), emin)
+                emax = max(eobs[det].max(), emax)
+
                 if save_los:
                     de["los"][e_offset : e_offset + num_det] = los
 
@@ -744,6 +780,12 @@ def _project_photons(
             for field in event_fields:
                 de[field].resize((n_events,))
 
+        flux *= erg_per_keV / exp_time / area
+
+        pe.create_dataset("flux", data=flux)
+        pe.create_dataset("emin", data=emin)
+        pe.create_dataset("emax", data=emax)
+
         fe.close()
 
     f.close()
