@@ -2,7 +2,7 @@ from numbers import Number
 
 import numpy as np
 from more_itertools import chunked
-from soxs.constants import abund_tables, atomic_weights, elem_names, metal_elem
+from soxs.constants import atomic_weights, elem_names, metal_elem
 from soxs.utils import parse_prng, regrid_spectrum
 from unyt.array import unyt_quantity
 from yt.data_objects.static_output import Dataset
@@ -15,7 +15,7 @@ from pyxsim.spectral_models import (
     MekalSpectralModel,
     TableCIEModel,
 )
-from pyxsim.utils import compute_H_abund, mylog, parse_value
+from pyxsim.utils import _parse_abund_table, compute_H_abund, mylog, parse_value
 
 
 class ThermalSourceModel(SourceModel):
@@ -34,6 +34,7 @@ class ThermalSourceModel(SourceModel):
         kT_max=64.0,
         var_elem=None,
         max_density=None,
+        min_entropy=None,
         method="invert_cdf",
         abund_table="angr",
         prng=None,
@@ -64,11 +65,18 @@ class ThermalSourceModel(SourceModel):
                     max_density = unyt_quantity(max_density[0], max_density[1])
                 else:
                     max_density = unyt_quantity(max_density, "g/cm**3")
+        if min_entropy is not None:
+            if not isinstance(min_entropy, unyt_quantity):
+                if isinstance(min_entropy, tuple):
+                    min_entropy = unyt_quantity(min_entropy[0], min_entropy[1])
+                else:
+                    min_entropy = unyt_quantity(min_entropy, "keV*cm**2")
         self.temperature_field = temperature_field
         self.emission_measure_field = emission_measure_field
         self.density_field = None  # Will be determined later
         self.nh_field = None  # Will be set by the subclass
         self.max_density = max_density
+        self.min_entropy = min_entropy
         self.tot_num_cells = 0  # Will be determined later
         self.ftype = "gas"
         self.binscale = binscale
@@ -85,7 +93,8 @@ class ThermalSourceModel(SourceModel):
         self.pbar = None
         self.Zconvert = 1.0
         self.mconvert = {}
-        self.atable = abund_tables[abund_table].copy()
+        self.abund_table = abund_table
+        self.atable = _parse_abund_table(abund_table)
         if h_fraction is None:
             h_fraction = compute_H_abund(abund_table)
         self.h_fraction = h_fraction
@@ -111,6 +120,7 @@ class ThermalSourceModel(SourceModel):
             "method": self.method,
             "model_vers": self.spectral_model.model_vers,
             "max_density": self.max_density,
+            "min_entropy": self.min_entropy,
             "abund_table": self.abund_table,
             "h_fraction": self.h_fraction,
             "var_elem": self.var_elem,
@@ -196,6 +206,7 @@ class ThermalSourceModel(SourceModel):
                 "Not all fields have the same field type! Fields used: %s", fields
             )
         self.density_field = (ftype, "density")
+        self.entropy_field = (ftype, "entropy")
         mylog.info("Using emission measure field '%s'.", self.emission_measure_field)
         mylog.info("Using temperature field '%s'.", self.temperature_field)
         if self.nh_field is not None:
@@ -256,7 +267,6 @@ class ThermalSourceModel(SourceModel):
         return self.spectral_model.make_fluxf(emin, emax, energy=energy)
 
     def process_data(self, mode, chunk, spectral_norm, fluxf=None):
-
         spec = np.zeros(self.nbins)
 
         orig_shape = chunk[self.temperature_field].shape
@@ -278,11 +288,23 @@ class ThermalSourceModel(SourceModel):
 
         if self.max_density is not None:
             cut &= np.ravel(chunk[self.density_field]) < self.max_density
+        if self.min_entropy is not None:
+            cut &= np.ravel(chunk[self.entropy_field]) > self.min_entropy
         kT = np.ravel(chunk[self.temperature_field].to_value("keV", "thermal"))
         cut &= (kT >= self.kT_min) & (kT <= self.kT_max)
 
         cell_nrm = np.ravel(chunk[self.emission_measure_field].d * spectral_norm)
 
+        if self.nh_field is not None:
+            nH = np.ravel(chunk[self.nh_field].d)
+        else:
+            nH = None
+
+        if isinstance(self.h_fraction, Number):
+            X_H = self.h_fraction
+        else:
+            X_H = np.ravel(chunk[self.h_fraction].d)
+
         num_cells = cut.sum()
 
         if mode in ["photons", "spectrum"]:
@@ -292,20 +314,26 @@ class ThermalSourceModel(SourceModel):
             else:
                 self.pbar.update(orig_ncells - num_cells)
         elif num_cells == 0:
+            # Here, we have no active cells, and so we
+            # return an array of zeros with the original shape.
+            # But yt needs to know that we may depend on the metallicity
+            # or abundance fields, so we check for that here. Very hacky!
+            if not isinstance(self.Zmet, Number):
+                chunk[self.Zmet]
+            if self.num_var_elem > 0:
+                elem_keys = self.var_ion_keys if self._nei else self.var_elem_keys
+                for key in elem_keys:
+                    value = self.var_elem[key]
+                    if not isinstance(value, Number):
+                        chunk[value]
             return np.zeros(orig_shape)
 
         kT = kT[cut]
         cell_nrm = cell_nrm[cut]
-
-        if self.nh_field is not None:
-            nH = np.ravel(chunk[self.nh_field].d)[cut]
-        else:
-            nH = None
-
-        if isinstance(self.h_fraction, Number):
-            X_H = self.h_fraction
-        else:
-            X_H = np.ravel(chunk[self.h_fraction].d)[cut]
+        if nH is not None:
+            nH = nH[cut]
+        if not isinstance(X_H, Number):
+            X_H = X_H[cut]
 
         if self._nei:
             metalZ = np.zeros(num_cells)
@@ -319,7 +347,7 @@ class ThermalSourceModel(SourceModel):
                 fac = self.Zconvert
                 if str(mZ.units) != "Zsun":
                     fac /= X_H
-                metalZ = np.ravel(mZ.d[cut] * fac)
+                metalZ = np.ravel(mZ.d * fac)[cut]
 
         elemZ = None
         if self.num_var_elem > 0:
@@ -333,7 +361,7 @@ class ThermalSourceModel(SourceModel):
                     fac = self.mconvert[key]
                     if str(eZ.units) != "Zsun":
                         fac /= X_H
-                    elemZ[j, :] = np.ravel(eZ.d[cut] * fac)
+                    elemZ[j, :] = np.ravel(eZ.d * fac)[cut]
 
         if self.observer == "internal" and mode == "photons":
             pos = np.array(
@@ -355,7 +383,6 @@ class ThermalSourceModel(SourceModel):
         idxs = np.where(cut)[0]
 
         for ck in chunked(range(num_cells), 100):
-
             ibegin = ck[0]
             iend = ck[-1] + 1
             nck = iend - ibegin
@@ -365,7 +392,6 @@ class ThermalSourceModel(SourceModel):
             kTi = kT[ibegin:iend]
 
             if mode in ["photons", "spectrum"]:
-
                 if self._density_dependence:
                     nHi = nH[ibegin:iend]
                     cspec, mspec, vspec = self.spectral_model.get_spectrum(kTi, nHi)
@@ -378,9 +404,9 @@ class ThermalSourceModel(SourceModel):
                     tot_spec += np.sum(
                         elemZ[:, ibegin:iend, np.newaxis] * vspec, axis=0
                     )
+                np.clip(tot_spec, 0.0, None, out=tot_spec)
 
                 if mode == "photons":
-
                     spec_sum = tot_spec.sum(axis=-1)
                     cell_norm = spec_sum * cnm
 
@@ -413,13 +439,11 @@ class ThermalSourceModel(SourceModel):
                     start_e = end_e
 
                 elif mode == "spectrum":
-
                     spec += np.sum(tot_spec * cnm[:, np.newaxis], axis=0)
 
                 self.pbar.update(nck)
 
             else:
-
                 if self._density_dependence:
                     nHi = nH[ibegin:iend]
                     cflux, mflux, vflux = fluxf(kTi, nHi)
@@ -517,6 +541,10 @@ class IGMSourceModel(ThermalSourceModel):
         The maximum density of the cells or particles to use when generating
         photons. If a float, the units are assumed to be g/cm**3.
         Default: None, meaning no maximum density.
+    min_entropy : float, (value, unit) tuple, :class:`~yt.units.yt_array.YTQuantity`, or :class:`~astropy.units.Quantity`
+        The minimum entropy of the cells or particles to use when generating
+        photons. If a float, the units are assumed to be keV*cm**2.
+        Default: None, meaning no minimum entropy.
     var_elem : dictionary, optional
         Elements that should be allowed to vary freely from the single abundance
         parameter. Each dictionary value, specified by the abundance symbol,
@@ -562,6 +590,7 @@ class IGMSourceModel(ThermalSourceModel):
         kT_min=0.00431,
         kT_max=64.0,
         max_density=None,
+        min_entropy=None,
         var_elem=None,
         method="invert_cdf",
         model_vers="4_lo",
@@ -593,6 +622,7 @@ class IGMSourceModel(ThermalSourceModel):
             nH_max=nH_max,
             var_elem=var_elem,
             max_density=max_density,
+            min_entropy=min_entropy,
             method=method,
             abund_table="feld",
             prng=prng,
@@ -655,6 +685,10 @@ class CIESourceModel(ThermalSourceModel):
         The maximum density of the cells or particles to use when generating
         photons. If a float, the units are assumed to be g/cm**3.
         Default: None, meaning no maximum density.
+    min_entropy : float, (value, unit) tuple, :class:`~yt.units.yt_array.YTQuantity`, or :class:`~astropy.units.Quantity`
+        The minimum entropy of the cells or particles to use when generating
+        photons. If a float, the units are assumed to be keV*cm**2.
+        Default: None, meaning no minimum entropy.
     var_elem : dictionary, optional
         Elements that should be allowed to vary freely from the single abundance
         parameter. Each dictionary value, specified by the abundance symbol,
@@ -731,6 +765,7 @@ class CIESourceModel(ThermalSourceModel):
         kT_min=0.025,
         kT_max=64.0,
         max_density=None,
+        min_entropy=None,
         var_elem=None,
         method="invert_cdf",
         thermal_broad=True,
@@ -789,6 +824,7 @@ class CIESourceModel(ThermalSourceModel):
             kT_max=kT_max,
             var_elem=var_elem,
             max_density=max_density,
+            min_entropy=min_entropy,
             method=method,
             abund_table=abund_table,
             prng=prng,
@@ -855,6 +891,10 @@ class NEISourceModel(CIESourceModel):
         The maximum density of the cells or particles to use when generating
         photons. If a float, the units are assumed to be g/cm**3.
         Default: None, meaning no maximum density.
+    min_entropy : float, (value, unit) tuple, :class:`~yt.units.yt_array.YTQuantity`, or :class:`~astropy.units.Quantity`
+        The minimum entropy of the cells or particles to use when generating
+        photons. If a float, the units are assumed to be keV*cm**2.
+        Default: None, meaning no minimum entropy.
     method : string, optional
         The method used to generate the photon energies from the spectrum:
         "invert_cdf": Invert the cumulative distribution function of the spectrum.
@@ -926,6 +966,7 @@ class NEISourceModel(CIESourceModel):
         kT_min=0.025,
         kT_max=64.0,
         max_density=None,
+        min_entropy=None,
         method="invert_cdf",
         thermal_broad=True,
         model_root=None,
@@ -947,6 +988,7 @@ class NEISourceModel(CIESourceModel):
             kT_min=kT_min,
             kT_max=kT_max,
             max_density=max_density,
+            min_entropy=min_entropy,
             var_elem=var_elem,
             method=method,
             thermal_broad=thermal_broad,
