@@ -1,6 +1,7 @@
 """
 Photon emission and absoprtion models.
 """
+
 import numpy as np
 from scipy.interpolate import interp1d
 from soxs.constants import K_per_keV
@@ -279,10 +280,16 @@ class Atable1DSpectralModel(ThermalSpectralModel):
     def prepare_spectrum(self, zobs):
         eidxs, ne, ebins, emid, de = self.sgen._get_energies(zobs)
         cosmic_spec, metal_spec, var_spec = self.sgen._get_table(ne, eidxs, zobs)
-        self.cosmic_spec = 1.0e-14 * regrid_spectrum(self.ebins, ebins, cosmic_spec)
-        self.metal_spec = 1.0e-14 * regrid_spectrum(self.ebins, ebins, metal_spec)
+        self.cosmic_spec = 1.0e-14 * regrid_spectrum(
+            self.ebins, ebins, cosmic_spec, clip_neg=False
+        )
+        self.metal_spec = 1.0e-14 * regrid_spectrum(
+            self.ebins, ebins, metal_spec, clip_neg=False
+        )
         if var_spec is not None:
-            var_spec = 1.0e-14 * regrid_spectrum(self.ebins, ebins, var_spec)
+            var_spec = 1.0e-14 * regrid_spectrum(
+                self.ebins, ebins, var_spec, clip_neg=False
+            )
         self.var_spec = var_spec
         self.si = SpectralInterpolator1D(
             self.Tvals, self.cosmic_spec, self.metal_spec, self.var_spec
@@ -418,10 +425,16 @@ class IGMSpectralModel(ThermalSpectralModel):
         """
         eidxs, ne, ebins, emid, de = self.igen._get_energies(zobs)
         cosmic_spec, metal_spec, var_spec = self.igen._get_table(ne, eidxs, zobs)
-        self.cosmic_spec = 1.0e-14 * regrid_spectrum(self.ebins, ebins, cosmic_spec)
-        self.metal_spec = 1.0e-14 * regrid_spectrum(self.ebins, ebins, metal_spec)
+        self.cosmic_spec = 1.0e-14 * regrid_spectrum(
+            self.ebins, ebins, cosmic_spec, clip_neg=False
+        )
+        self.metal_spec = 1.0e-14 * regrid_spectrum(
+            self.ebins, ebins, metal_spec, clip_neg=False
+        )
         if var_spec is not None:
-            var_spec = 1.0e-14 * regrid_spectrum(self.ebins, ebins, var_spec)
+            var_spec = 1.0e-14 * regrid_spectrum(
+                self.ebins, ebins, var_spec, clip_neg=False
+            )
         self.var_spec = var_spec
         self.cie_model.prepare_spectrum(zobs)
         self.si = SpectralInterpolator2D(
@@ -593,6 +606,7 @@ class TBabsModel(AbsorptionModel):
     --------
     >>> tbabs_model = TBabsModel(0.1)
     """
+
     _name = "tbabs"
 
     def __init__(self, nH, abund_table="angr"):
@@ -618,6 +632,7 @@ class WabsModel(AbsorptionModel):
     --------
     >>> wabs_model = WabsModel(0.1)
     """
+
     _name = "wabs"
 
     def __init__(self, nH, abund_table="angr"):
