@@ -4,7 +4,6 @@ import pytest
 
 
 def pytest_addoption(parser):
-    parser.addoption("--answer_dir", help="Directory where answers are stored.")
     parser.addoption("--check_dir", help="Directory where spectrum checks are stored.")
     parser.addoption(
         "--answer_store",
@@ -18,17 +17,6 @@ def answer_store(request):
     return request.config.getoption("--answer_store")
 
 
-@pytest.fixture()
-def answer_dir(request):
-    ad = request.config.getoption("--answer_dir")
-    if ad is None:
-        pytest.skip("No answer directory")
-    ad = os.path.abspath(ad)
-    if not os.path.exists(ad):
-        os.makedirs(ad)
-    return ad
-
-
 @pytest.fixture()
 def check_dir(request):
     cd = request.config.getoption("--check_dir")
