@@ -21,13 +21,26 @@ if [[ ${mode} == "testing" ]]; then
     echo "[yt]" > $HOME/.config/yt/yt.toml
     echo "test_data_dir = \"${PWD}\"" >> $HOME/.config/yt/yt.toml
     cat $HOME/.config/yt/yt.toml
+
+    # Set location of SOXS data
+    mkdir -p $HOME/.config/soxs
+    echo "[soxs]" > $HOME/.config/soxs/soxs.cfg
+    echo "soxs_data_dir = ${GITHUB_WORKSPACE}/soxs_data" >> $HOME/.config/soxs/soxs.cfg
+    echo "soxs_answer_dir = ${GITHUB_WORKSPACE}/${ANSWER_VER}" >> $HOME/.config/soxs/soxs.cfg
+    cat $HOME/.config/soxs/soxs.cfg
+
 fi
 
 # Install dependencies using conda
 
-PYVER=`python --version`
+# Install dependencies using mamba and pip
+
+eval "$(micromamba shell hook --shell bash)"
+micromamba shell init --shell bash --root-prefix=~/micromamba
+micromamba activate test-env
+
+micromamba install --yes -c conda-forge numpy pytest pip astropy scipy cython h5py yt
 
-conda install --yes numpy pytest pip h5py astropy nose cython scipy yt
 git clone https://github.com/lynx-x-ray-observatory/soxs
 cd soxs
 pip install .
